////////////////////////////
//                        //
// ACGI++ Class Hierarchy //
//                        //
///////////////////////////////////////////////////////////////
// a_array.cpp                                               //
//                                                           //
// Classes to be used in CGI development using C++           //
//                                                           //
// Copyright of Quasit Technologies, Inc.                    //
//                                                           //
///////////////////////////////////////////////////////////////
#include "a_acgi.h"

#pragma message( "Compiling " __FILE__ )

#ifdef _DEBUG_DUMP_
void AArray::dump(void)
{
  cout << endl << "---START: AArray::dump()---";

  ABaseElement::dump();                                 //a_Do parent dump as well (calls the doOut virtual member!)

  cout << "m_pData = " << (void *)m_pData;
  cout << "  m_iSize = " << m_iSize;
  cout << "  m_iAllocSize = " << m_iAllocSize << endl;

  cout << "---END: AArray::dump()---" << endl;
}
#endif

//------------------------------------------------------------------------
AArray::AArray(AArray *paSource)
{
  if (paSource)
  {
    //a_Invoke the copy member
    _bCopy(*paSource);
  }
  else
  {
    //a_Default constructor
    m_pData = NULL;
    m_iSize = 0x0;              //a_Array size
    m_iAllocSize = 0x0;         //a_Space allocated in BYTEs
  }

  m_bSafetyBYTE = 0x0;          //a_If this is non zero, then you may have tried writing to a NULL/inavlid array/offset
}

AArray::~AArray()
{
  delete []m_pData;
  m_pData = NULL;
  m_iSize = 0x0;
  m_iAllocSize = 0x0;
}

void AArray::_bCopy(const AArray &aSource)
{
  ABaseElement::_bCopy(aSource);   //a_Copy parent first
  
  //a_Assign this to a BYTE array
  _aSetToBYTEArray(aSource.aGetArray(), aSource.aGetAllocSize());
}

void AArray::aSetBYTEAt(int iBYTEPos, BYTE bMask, int iMode)
{
  assert(iBYTEPos >= 0x0 && iBYTEPos < m_iAllocSize);

  switch (iMode)
  {
    case BLIT_COPY :
      m_pData[iBYTEPos] = bMask;
    break;

    case BLIT_AND :
      m_pData[iBYTEPos] &= bMask;
    break;

    case BLIT_OR :
      m_pData[iBYTEPos] |= bMask;
    break;

    case BLIT_XOR :
      m_pData[iBYTEPos] ^= bMask;
    break;

    default : assert(0x0);        //a_Unknown mode
 }
}

void AArray::_aSetToBYTEArray(const BYTE *pcbSource, int iLength)
{
  if (iLength > 0x0)
  {
    //a_If parameters are valid make a copy, else it just clears it
    BYTE *pNewData = aMemDup(pcbSource, iLength);
    if (pNewData)
    {
      m_iAllocSize = iLength;              //a_Assign size if successfully allocated
      m_iSize = _aMapAllocToSize(iLength);  //a_Mapped size

      delete []m_pData;
      m_pData = pNewData;
    }
  }

  //a_Returns if length is invalid or could not allocate buffer
  //a_ it preserves the old buffer if something fails
}

void AArray::aBlit(int iMyStart, AArray &aSource, int iSourceStart, int iLength, int iMode)
{
  int iCount = iLength;
  if (iLength == -1)
  {
    iCount = __min(iLength, m_iSize);             //a_Can't be longer than this
    iCount = __min(iCount - iMyStart, aSource.m_iSize);      //a_Constrain to destination
  }

  //a_Might be a bit inefficient but polymorphic :)
  for (register int iX = 0x0; iX < iCount; iX++)
  {
    aSetAt(iMyStart + iX, aSource.aGetAt(iSourceStart + iX), iMode);
  }
}

void AArray::aSetRange(int iState, int iMode, int iStart, int iEnd)
{
  if (iEnd = -1 || iEnd > aGetSize()) iEnd = aGetSize();
  if (!m_pData || (iStart > iEnd))
  {
    assert(0x0);  //a_Either array is NULL or start > end (maybe (start||end > allocsize)?)
    return;
  }

  //a_aSetAt is a virtual member and the child should know how to handle iState
  for (register int iX = iStart; iX < iEnd; iX++)
    aSetAt(iX, iState, iMode);
}

void AArray::_doArrayOutput(AStreamOutput *pasOut, int iPBM) const
{
  assert(pasOut);
  assert((aGetSize() % 0x8) == 0x0);     //a_Multiples of BYTEs only for proper output!

  if (m_pData)
  {
    int iEnd = aGetTrueSize() - 0x1, iX;
    BYTE bX;
    for (iX = 0x0; iX < iEnd; iX++)
    {
      //a_The BYTE is mirrored/reveresed if PBM is the output
      bX = (iPBM ? g_aCrypto.convertoMirrorBYTE(_aGetBYTEValue(iX)) : _aGetBYTEValue(iX)); 

      if (bX < 0x10)
        *pasOut << "0x0";
      else
        *pasOut << "0x"; 
      
      *pasOut << hex << int(bX);

      if (((iX + 1) % 0x8) == 0x0)
        *pasOut << "," << endl;
      else
        *pasOut << ", ";
    }

    //a_Do the last set, since no comma a the end is needed
    bX = (iPBM ? g_aCrypto.convertoMirrorBYTE(_aGetBYTEValue(iX)) : _aGetBYTEValue(iX)); 
    
    if (bX < 0x10)
      *pasOut << "0x0";
    else
      *pasOut << "0x"; 
    
    *pasOut << hex << int(bX);   //a_NOTE: No last endl!
  }
}

int AArray::aSetSize(int iNewSize, int iRealloc)
{
  assert(iNewSize > 0x0 && iNewSize < INT_MAX);

  if (iNewSize > aGetAllocSize() || iRealloc || !m_pData)
  {
    //a_Need to reallocate and reassign from original
    _aSetToBYTEArray(aGetArray(), iNewSize);
  }
  else
  {
    //a_Just shrink, without reallocating
    m_iSize = _aMapAllocToSize(iNewSize); 
    return 0x1;
  }

  return 0x0;               //a_Failed
}
